@@ -84,7 +84,7 @@ static int chunk_mux_init(AVFormatContext *s)
     return 0;
 }
 
-static int get_chunk_filename(AVFormatContext *s, int is_header, char filename[MAX_FILENAME_SIZE])
+static int get_chunk_filename(AVFormatContext *s, int is_header, char *filename)
 {
     WebMChunkContext *wc = s->priv_data;
     AVFormatContext *oc = wc->avf;
@@ -92,20 +92,15 @@ static int get_chunk_filename(AVFormatContext *s, int is_header, char filename[M
         return AVERROR(EINVAL);
     }
     if (is_header) {
-        int len;
         if (!wc->header_filename) {
             av_log(oc, AV_LOG_ERROR, "No header filename provided\n");
             return AVERROR(EINVAL);
         }
-        len = av_strlcpy(filename, wc->header_filename, MAX_FILENAME_SIZE);
-        if (len >= MAX_FILENAME_SIZE) {
-            av_log(oc, AV_LOG_ERROR, "Header filename too long\n");
-            return AVERROR(EINVAL);
-        }
+        av_strlcpy(filename, wc->header_filename, strlen(wc->header_filename) + 1);
     } else {
         if (av_get_frame_filename(filename, MAX_FILENAME_SIZE,
-                                  s->filename, wc->chunk_index - 1) < 0) {
-            av_log(oc, AV_LOG_ERROR, "Invalid chunk filename template '%s'\n", s->filename);
+                                  s->url, wc->chunk_index - 1) < 0) {
+            av_log(oc, AV_LOG_ERROR, "Invalid chunk filename template '%s'\n", s->url);
             return AVERROR(EINVAL);
         }
     }
@@ -124,7 +119,7 @@ static int webm_chunk_write_header(AVFormatContext *s)
     if (s->nb_streams != 1) { return AVERROR_INVALIDDATA; }
 
     wc->chunk_index = wc->chunk_start_index;
-    wc->oformat = av_guess_format("webm", s->filename, "video/webm");
+    wc->oformat = av_guess_format("webm", s->url, "video/webm");
     if (!wc->oformat)
         return AVERROR_MUXER_NOT_FOUND;
 
@@ -132,12 +127,12 @@ static int webm_chunk_write_header(AVFormatContext *s)
     if (ret < 0)
         return ret;
     oc = wc->avf;
-    ret = get_chunk_filename(s, 1, oc->filename);
+    ret = get_chunk_filename(s, 1, oc->url);
     if (ret < 0)
         return ret;
     if (wc->http_method)
         av_dict_set(&options, "method", wc->http_method, 0);
-    ret = s->io_open(s, &oc->pb, oc->filename, AVIO_FLAG_WRITE, &options);
+    ret = s->io_open(s, &oc->pb, oc->url, AVIO_FLAG_WRITE, &options);
     av_dict_free(&options);
     if (ret < 0)
         return ret;
@@ -167,7 +162,7 @@ static int chunk_start(AVFormatContext *s)
     return 0;
 }
 
-static int chunk_end(AVFormatContext *s, int flush)
+static int chunk_end(AVFormatContext *s)
 {
     WebMChunkContext *wc = s->priv_data;
     AVFormatContext *oc = wc->avf;
@@ -178,14 +173,11 @@ static int chunk_end(AVFormatContext *s, int flush)
     char filename[MAX_FILENAME_SIZE];
     AVDictionary *options = NULL;
 
-    if (!oc->pb)
+    if (wc->chunk_start_index == wc->chunk_index)
         return 0;
-
-    if (flush)
-        // Flush the cluster in WebM muxer.
-        oc->oformat->write_packet(oc, NULL);
+    // Flush the cluster in WebM muxer.
+    oc->oformat->write_packet(oc, NULL);
     buffer_size = avio_close_dyn_buf(oc->pb, &buffer);
-    oc->pb = NULL;
     ret = get_chunk_filename(s, 0, filename);
     if (ret < 0)
         goto fail;
@@ -196,6 +188,7 @@ static int chunk_end(AVFormatContext *s, int flush)
         goto fail;
     avio_write(pb, buffer, buffer_size);
     ff_format_io_close(s, &pb);
+    oc->pb = NULL;
 fail:
     av_dict_free(&options);
     av_free(buffer);
@@ -217,19 +210,27 @@ static int webm_chunk_write_packet(AVFormatContext *s, AVPacket *pkt)
     }
 
     // For video, a new chunk is started only on key frames. For audio, a new
-    // chunk is started based on chunk_duration. Also, a new chunk is started
-    // unconditionally if there is no currently open chunk.
-    if (!oc->pb || (st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO &&
+    // chunk is started based on chunk_duration.
+    if ((st->codecpar->codec_type == AVMEDIA_TYPE_VIDEO &&
          (pkt->flags & AV_PKT_FLAG_KEY)) ||
         (st->codecpar->codec_type == AVMEDIA_TYPE_AUDIO &&
-         wc->duration_written >= wc->chunk_duration)) {
+         (pkt->pts == 0 || wc->duration_written >= wc->chunk_duration))) {
         wc->duration_written = 0;
-        if ((ret = chunk_end(s, 1)) < 0 || (ret = chunk_start(s)) < 0) {
-            return ret;
+        if ((ret = chunk_end(s)) < 0 || (ret = chunk_start(s)) < 0) {
+            goto fail;
         }
     }
 
     ret = oc->oformat->write_packet(oc, pkt);
+    if (ret < 0)
+        goto fail;
+
+fail:
+    if (ret < 0) {
+        oc->streams = NULL;
+        oc->nb_streams = 0;
+        avformat_free_context(oc);
+    }
 
     return ret;
 }
@@ -238,20 +239,12 @@ static int webm_chunk_write_trailer(AVFormatContext *s)
 {
     WebMChunkContext *wc = s->priv_data;
     AVFormatContext *oc = wc->avf;
-    int ret;
-
-    if (!oc->pb) {
-        ret = chunk_start(s);
-        if (ret < 0)
-            goto fail;
-    }
     oc->oformat->write_trailer(oc);
-    ret = chunk_end(s, 0);
-fail:
+    chunk_end(s);
     oc->streams = NULL;
     oc->nb_streams = 0;
     avformat_free_context(oc);
-    return ret;
+    return 0;
 }
 
 #define OFFSET(x) offsetof(WebMChunkContext, x)
